//+------------------------------------------------------------------+
//|                                            elliot oscillator.mq4 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1  DeepSkyBlue
#property indicator_color2  DimGray
#property indicator_color3  DimGray
#property indicator_color4  Gold
#property indicator_color5  PaleVioletRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_width4  2
#property indicator_width5  2

//
//
//
//
//

extern int    shortPeriod =  5;
extern int    longPeriod  = 35;
extern int    peakCount   =  5;
extern string identifier  = "elliotWaveLines";
extern color  linesColor  = Silver;

//
//
//
//
//

double ellBuffer[];
double mauBuffer[];
double madBuffer[];
double peakUp[];
double peakDn[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,ellBuffer); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,mauBuffer);
   SetIndexBuffer(2,madBuffer);
   SetIndexBuffer(3,peakUp);  SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,peakDn);  SetIndexStyle(4,DRAW_HISTOGRAM);
   IndicatorShortName("Elliot oscillator ( "+shortPeriod+","+longPeriod+")");
   return(0);
}
int deinit() { deleteLines(); return(0); }

//
//
//
//
//

int start()
{
   double alpha     = 2.0/(1.0+longPeriod+MathCeil(shortPeriod/2.0));
   int counted_bars = IndicatorCounted();
   int limit,i,k;

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-longPeriod);
           
   //
   //
   //
   //
   //

   for(i = limit; i >= 0; i--)
   {
      ellBuffer[i] = iMA(NULL,0,shortPeriod,0,MODE_SMA,PRICE_MEDIAN,i)-iMA(NULL,0,longPeriod,0,MODE_SMA,PRICE_MEDIAN,i);

         if (mauBuffer[i+1]==EMPTY_VALUE) if (ellBuffer[i]>0) mauBuffer[i+1] = ellBuffer[i]; else  mauBuffer[i+1] = 0;
         if (madBuffer[i+1]==EMPTY_VALUE) if (ellBuffer[i]<0) madBuffer[i+1] = ellBuffer[i]; else  madBuffer[i+1] = 0;
            
      madBuffer[i] = madBuffer[i+1];
      mauBuffer[i] = mauBuffer[i+1];
         
      //
      //
      //
      //
      //
         
      if (ellBuffer[i] < 0) madBuffer[i] = madBuffer[i+1]+alpha*(ellBuffer[i]-madBuffer[i+1]);
      if (ellBuffer[i] > 0) mauBuffer[i] = mauBuffer[i+1]+alpha*(ellBuffer[i]-mauBuffer[i+1]);
   }
   
   //
   //
   //
   //
   //

      deleteLines();   
      for (i=0,k=0; i<Bars && k<peakCount+1; i++) 
      {
         if (peakUp[i] != EMPTY_VALUE) { peakUp[i] = EMPTY_VALUE; k++; }
         if (peakDn[i] != EMPTY_VALUE) { peakDn[i] = EMPTY_VALUE; k++; }
      }
     
      //
      //
      //
      //
      //
       
      int      direction     = 0;   
      int      startFrom     = 0;
      double   lastPeakPrice = 0;
      datetime lastPeakTime  = 0;
      
      //
      //
      //
      //
      //
      
      for (i=0,k=0; i<Bars && k<peakCount+1; i++) 
      {
         if (ellBuffer[i] > 0 && ellBuffer[i]>mauBuffer[i])
         {
            if (direction < 0) { markLow(startFrom,i,lastPeakPrice,lastPeakTime); startFrom = i; k++; }
                direction = 1;
                
         }
         if (ellBuffer[i] < 0 && ellBuffer[i]<madBuffer[i])
         {
            if (direction > 0) { markHigh(startFrom,i,lastPeakPrice,lastPeakTime); startFrom = i; k++; }
                direction = -1;
         }
      }   
      
   //
   //
   //
   //
   //
   
   return(0);      
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void markLow(int start, int end, double& lastPeakPrice, datetime& lastPeakTime)
{
   int peakAt = ArrayMinimum(Low,end-start,start); peakDn[peakAt] = ellBuffer[peakAt];
   
   //
   //
   //
   //
   //
   
   if (lastPeakPrice!=0) drawLine(lastPeakPrice,lastPeakTime,Low[peakAt],Time[peakAt]);
       lastPeakPrice = Low[peakAt];
       lastPeakTime  = Time[peakAt];
}
void markHigh(int start, int end, double& lastPeakPrice, datetime& lastPeakTime)
{
   int peakAt = ArrayMaximum(High,end-start,start); peakUp[peakAt] = ellBuffer[peakAt];
   
   //
   //
   //
   //
   //
   
   if (lastPeakPrice!=0) drawLine(lastPeakPrice,lastPeakTime,High[peakAt],Time[peakAt]);
       lastPeakPrice = High[peakAt];
       lastPeakTime  = Time[peakAt];
}

//
//
//
//
//

void drawLine(double startPrice, datetime startTime, double endPrice, datetime endTime)
{
   string name = identifier+":"+startTime;
      ObjectCreate(name,OBJ_TREND,0,startTime,startPrice,endTime,endPrice);
         ObjectSet(name,OBJPROP_STYLE,STYLE_DOT);
         ObjectSet(name,OBJPROP_RAY,false);
         ObjectSet(name,OBJPROP_COLOR,linesColor);
}
void deleteLines()
{
   string lookFor = identifier+":";
   for (int i=ObjectsTotal(); i>=0; i--)
      {
         string name = ObjectName(i);
         if (StringFind(name,lookFor)==0) ObjectDelete(name);
      }
}